#include <stdint.h>
#include <avr/io.h>
#include <avr/delay.h>

void lcd_enable() {
	/* short burst on 'display enable' to have the lcd read the data on
	the ports + wait for recovery */
	PORTD |= _BV(PD5);
	_delay_us(20);
	PORTD &= ~_BV(PD5);
	_delay_ms(2);
}

void lcd_data(uint8_t byte) {
	uint8_t upper, lower;
	upper = (byte >> 4) | _BV(PD4);
	lower = (byte & 0x0f) | _BV(PD4);
	PORTD = upper;
	lcd_enable();
	PORTD = lower;
	lcd_enable();
}

void lcd_command(uint8_t byte) {
	uint8_t upper, lower;
	upper = byte >> 4;
	lower = byte & 0x0f;
	PORTD = upper;
	lcd_enable();
	PORTD = lower;
	lcd_enable();
}

void lcd_init() {
	DDRD = 0xff; 
	_delay_ms(100); /* power-on wait */
	PORTD = 0x06; /* reset */
	lcd_enable();
	lcd_enable();
	lcd_enable();
	PORTD = 0x02; /* enable 4bit */
	lcd_enable();
	lcd_command(0x2e); /* + 2 lines */
	lcd_command(0x0c); /* display on */
	lcd_command(0x01); /* clear screen */
	lcd_command(0x02); /* increment mode */
}

void lcd_print(char *str) {
	while(*str)
		lcd_data(*str++);
}
